<?php
namespace Zeedhi\PaymentPlatform\Implementations\MundiPagg;

use Zeedhi\PaymentPlatform\BankSlip\BankSlipPlatform;
use Zeedhi\PaymentPlatform\BankSlip\BankSlipTransaction;

use Zeedhi\PaymentPlatform\InstantBuy\InstantBuyPlatform;
use Zeedhi\PaymentPlatform\InstantBuy\InstantBuyTransaction;

use Zeedhi\PaymentPlatform\NonInstantBuy\NonInstantBuyPlatform;
use Zeedhi\PaymentPlatform\NonInstantBuy\NonInstantBuyTransaction;

use Zeedhi\PaymentPlatform\Consult\ConsultPlatform;
use Zeedhi\PaymentPlatform\Consult\Consult;

use Zeedhi\PaymentPlatform\Revoke\RevokePlatform;
use Zeedhi\PaymentPlatform\Revoke\Revoke;

use Zeedhi\PaymentPlatform\CreditCard;
use Zeedhi\PaymentPlatform\Customer;
use Zeedhi\PaymentPlatform\RequestURL;

use Zeedhi\PaymentPlatform\Exception;

class PaymentImpl implements BankSlipPlatform, InstantBuyPlatform, NonInstantBuyPlatform, ConsultPlatform, RevokePlatform{

    const ENDPOINT = "https://api.mundipagg.com/core/v1/";
    
    const POST = "POST";
    const GET = "GET";
    const DELETE = "DELETE";
    
    /** @var String */
    private $merchantKey;
    /** @var array */
    private $requestHeaders;
    /** @var HttpRequest */
    private $httpRequest;

    /**
     * PaymentImpl constructor.
     *
     * @param String        $merchantKey
     * @param HttpRequest   $httpRequest
     */
    public function __construct($merchantKey, HttpRequest $httpRequest) {
        $this->merchantKey = $merchantKey;
        $this->httpRequest = $httpRequest;
        $this->requestHeaders = array(
            'Content-Type:application/json',
            'Accept: application/json',
            'Authorization: Basic '. base64_encode($this->merchantKey . ':')
        );
    }
    
    /**
     * @param String $type
     * 
     * @throws Exception unsupportedCustomerType
     * 
     * @return string
     */
    private function decodeTypeCustomer($type) {
        switch($type) {
            case Customer::INDIVIDUAL:
                return Customer::INDIVIDUAL;
            case Customer::COMPANY:
                return Customer::COMPANY;
            default:
                throw Exception::unsupportedCustomerType();
                break;
        }
    }
    
    /**
     * @param String $baseURL
     * 
     * @throws Exception unsupportedBaseURL
     * 
     * @return string
     */
    private function decodeBaseURL($baseURL) {
        switch($baseURL) {
            case RequestURL::MUNDIPAGG_CUSTOMERS:
                return RequestURL::MUNDIPAGG_CUSTOMERS;
            case RequestURL::MUNDIPAGG_CARDS:
                return RequestURL::MUNDIPAGG_CARDS;
            case RequestURL::MUNDIPAGG_CHARGES:
                return RequestURL::MUNDIPAGG_CHARGES;
            default:
                throw Exception::unsupportedBaseURL();
                break;
        }
    }
    
    /**
     * @param String $bank
     * 
     * @throws Exception unsupportedBank
     * 
     * @return string
     */
    private function decodeBank($bank) {
        switch($bank) {
            case BankSlipTransaction::BANCO_DO_BRASIL:
                return BankSlipTransaction::BANCO_DO_BRASIL;
            case BankSlipTransaction::SANTANDER:
                return BankSlipTransaction::SANTANDER;
            case BankSlipTransaction::BRADESCO:
                return BankSlipTransaction::BRADESCO;
            case BankSlipTransaction::ITAU:
                return BankSlipTransaction::ITAU;
            case BankSlipTransaction::HSBC:
                return BankSlipTransaction::HSBC;
            default:
                throw Exception::unsupportedBank();
                break;
        }
    }
    
    /**
     * @param cURL $request
     *
     * @return array $response
     * 
     */
    private function responseTreatment() {
        $response = json_decode($this->httpRequest->execute(), true);
        $httpStatusCode = $this->httpRequest->getInfo(CURLINFO_HTTP_CODE);
    
        $this->httpRequest->close();
        
        if($httpStatusCode == '200') {
            return $response;
        } else {
            throw Exception::apiMundipagg($httpStatusCode, $response["message"]);
        }
    }

    /**
     * @param String $url
     * 
     */
    private function configCurl($url) {
        $this->httpRequest->setUrl(self::ENDPOINT.$url);
        $this->httpRequest->setOption(CURLOPT_HTTPHEADER, $this->requestHeaders);
        $this->httpRequest->setOption(CURLOPT_RETURNTRANSFER, true);
    }

    /**
     * @param String $url
     * 
     * @return array 
     */
    private function createGetRequest($url) {
        $this->configCurl($url);
        $this->httpRequest->setOption(CURLOPT_CUSTOMREQUEST, self::GET);
        return $this->responseTreatment();
    }

    /**
     * @param String $url
     * @param String $verb
     * @param String $fields
     * 
     * @return array
     */
    private function createRequest($url, $verb, $fields){
        $this->configCurl($url);
        $this->httpRequest->setOption(CURLOPT_CUSTOMREQUEST, $verb);
        $this->httpRequest->setOption(CURLOPT_POSTFIELDS, json_encode($fields));
        return $this->responseTreatment();
    }
    
    /**
     * @param Customer $customer
     *
     * @return array
     */
    private function configCustomer(Customer $customer) {
        $request = array("name" => $customer->getName());
        
        if($customer->getEmail() !== NULL) {
            $request["email"] = $customer->getEmail();
        }
        
        if($customer->getAddress() !== NULL) {
            $address = array(
                "street" => $customer->getAddress()->getStreet(),
                "number" => $customer->getAddress()->getNumber(),
                "complement" => $customer->getAddress()->getComplement(),
                "zip_code" => $customer->getAddress()->getZipCode(),
                "neighborhood" => $customer->getAddress()->getNeighborhood(),
                "city" => $customer->getAddress()->getCity(),
                "state" => $customer->getAddress()->getState(),
                "country" => $customer->getAddress()->getCountry()
            );
            $request["address"] = $address;
        }
                
        if($customer->getDocument() !== NULL) {
            $request["document"] = $customer->getDocument();
            $request["type"] = $this->decodeTypeCustomer($customer->getType());
        }
        
        if($customer->getPhone() !== NULL) {
            $phone = array("phone" => $customer->getPhone());
            $request["metadata"] = $phone;
        }
        return $request;
    }

    /**
     * @param CreditCard $card
     * @param Customer $customer
     *
     * @return array
     */
    public function createToken(CreditCard $card, Customer $customer) {
        if($customer->getId() !== NULL) {
            $customerId = $customer->getId();
        } else {
            $request = $this->configCustomer($customer);
            $response = $this->createRequest(RequestURL::MUNDIPAGG_CUSTOMERS, self::POST, $request);
            $customerId = $response["id"];
        }
        
        $request = array(
            "number" => $card->getNumbers(),
            "holder_name" => $card->getHolderName(),
            "exp_month" => $card->getExpDateMonth(),
            "exp_year" => $card->getExpDateYear(),
            "cvv" => $card->getCvv(),
            "options" => array(
                "verify_card" => true
            )
        );
        
        $url = RequestURL::MUNDIPAGG_CUSTOMERS . '/' . $customerId . '/' . RequestURL::MUNDIPAGG_CARDS;
        $responseData = $this->createRequest($url, self::POST, $request);
        return array(
            "InstantBuyKey" => $responseData["id"],
            "CustomerId" => $customerId
        );
    }
    
    /**
     * @param InstantBuyTransaction $instantBuyTransaction
     *
     * @return string
     */
    public function instantBuyTransaction(InstantBuyTransaction $instantBuyTransaction){
        $request = array(
            "amount" => $instantBuyTransaction->getValue(),
            "payment" => array(
                "payment_method" => "credit_card",
                "credit_card" => array(
                    "installments" => $instantBuyTransaction->getNrInstallment(),
                    "card_id" => $instantBuyTransaction->getCreditCardToken()
                )
            ),
            "metadata" => array(
                "transactionId" => $instantBuyTransaction->getTransactionId()
            )
        );
        
        if($instantBuyTransaction->getCustomer()->getId() === NULL) {
            $request["customer"] = $this->configCustomer($instantBuyTransaction->getCustomer());
        } else {
            $request["customer_id"] = $instantBuyTransaction->getCustomer()->getId();
        }
        $responseData = $this->createRequest(RequestURL::MUNDIPAGG_CHARGES, self::POST, $request);
        return $responseData["id"];
    }
    
    /**
     * @param NonInstantBuyTransaction $nonInstantBuyTransaction
     *
     * @return string
     */
    public function nonInstantBuyTransaction(NonInstantBuyTransaction $nonInstantBuyTransaction){
        $request = array(
            "amount" => $nonInstantBuyTransaction->getValue(),
            "payment" => array(
                "payment_method" => "credit_card",
                "credit_card" => array(
                    "installments" => $nonInstantBuyTransaction->getNrInstallment(),
                    "card" => array(
                        "number" => $nonInstantBuyTransaction->getCreditCard()->getNumbers(),
                        "holder_name" => $nonInstantBuyTransaction->getCreditCard()->getHolderName(),
                        "exp_month" => $nonInstantBuyTransaction->getCreditCard()->getExpDateMonth(),
                        "exp_year" => $nonInstantBuyTransaction->getCreditCard()->getExpDateYear(),
                        "cvv" => $nonInstantBuyTransaction->getCreditCard()->getCvv(),
                    )
                )
            ),
            "metadata" => array(
                "transactionId" => $nonInstantBuyTransaction->getTransactionId()
            )
        );
        
        if($nonInstantBuyTransaction->getCustomer()->getId() === NULL) {
            $request["customer"] = $this->configCustomer($nonInstantBuyTransaction->getCustomer());
        } else {
            $request["customer_id"] = $nonInstantBuyTransaction->getCustomer()->getId();
        }
        $responseData = $this->createRequest(RequestURL::MUNDIPAGG_CHARGES, self::POST, $request);
        return $responseData["id"];
    }
    
    /**
     * @param BankSlip $bankSlipTransaction
     *
     * @return array
     */
    public function bankSlipTransaction(BankSlipTransaction $bankSlipTransaction){
        $daysToAddInBankSlipExpirationDate = date('Y-m-d H:i:s', strtotime("+" . $bankSlipTransaction->getDaysToAddInBankSlipExpirationDate() . " days"));
        $formatDate = explode(" ", $daysToAddInBankSlipExpirationDate);
        
        $request = array(
            "amount" => $bankSlipTransaction->getValue(),
            "payment" => array(
                "payment_method" => "boleto",
                "boleto" => array(
                    "bank" => $this->decodeBank($bankSlipTransaction->getBank()),
                    "instructions" =>$bankSlipTransaction->getInstructions(),
                    "due_at" => $formatDate[0].'T'.$formatDate[1].'Z'
                )
            ),
            "metadata" => array(
                "transactionId" => $bankSlipTransaction->getTransactionId(),
                "phone" => $bankSlipTransaction->getCustomer()->getPhone()
            )
        );
        
        if($bankSlipTransaction->getCustomer()->getId() === NULL) {
            $request["customer"] = $this->configCustomer($bankSlipTransaction->getCustomer());
        } else {
            $request["customer_id"] = $bankSlipTransaction->getCustomer()->getId();
        }
        
        $responseData = $this->createRequest(RequestURL::MUNDIPAGG_CHARGES, self::POST, $request);
        return array(
            "TransactionKey" => $responseData["id"],
            "BoletoUrl" => $responseData["last_transaction"]["url"]
        );
        
    }

    public function captureTransaction($url, $paymentId){
        
    }
    
    /**
     * @param Revoke $revoke
     *
     * @return string
     */
    public function revoke(Revoke $revoke) {
        
        $request = array();
        
        if($revoke->getValueToCancel() !== NULL) {
            $request["amount"] = $revoke->getValueToCancel();
        }
        
        $url = $this->decodeBaseURL($revoke->getBaseURL()) . '/' . $revoke->getId();
        $responseData = $this->createRequest($url, self::DELETE, $request);
        return $responseData["id"];
    }
    
    /**
     * @param Consult $consult
     *
     * @return array
     */
    public function consult(Consult $consult) {
        $url = $this->decodeBaseURL($consult->getBaseUrl()) . '/' . $consult->getId();
        $responseData = $this->createGetRequest($url);
        return $responseData;
    }
    
    /**
     * @param String $merchantKey
     *
     * @return PaymentImpl
     */
    public static function factory($merchantKey) {
        return new self($merchantKey, new HttpRequest);
    }
    
}