/**
* Creates an instance of ScreenService
* @class
* @name ScreenService
* @param {TemplateManager} templateManager - An instance of TemplateManager
* @param {$RootScope} $rootScope - A reference to the root scope of the project
* @param {I18nEngine} I18nEngine - An instance of I18nEngine
* @param {ZHPromise} ZHPromise - An instance of ZHPromise
* @param {RestEngine} restEngine - An instance of RestEngine
*/
function ScreenService(templateManager, $rootScope, I18nEngine, ZHPromise, restEngine) {
	var self = this;

	//Constants
	this.POPUP = "popup";
	this.SWIPE = "swipe";
	this.FILTER = "filter";

	/**
	* Changes the languange of the project
	* @function setLanguage
	* @memberof ScreenService
	* @instance
	* @params {String} language - New language to be set. Example: "en_us"
	* @returns {Promise} Returns a promise. On success the response provides the loaded words file for the language, and on reject it will provide the error data
	*/
	this.setLanguage = function(language) {
		return I18nEngine.setLanguage(language);
	};

	/**
	* Translates the value to the desired language
	* @function i18n
	* @memberof ScreenService
	* @instance
	* @param {String} message - Text to be translated
	* @param {Object} variables - Variables contained in the text that should also be translated
	* @returns {String} Translated text
	*/
	this.i18n = function(message, variables) {
		return I18nEngine.i18n(message, variables);
	};

	var footer;

	templateManager.projectPromise.then(function(container) {
		footer = container.zeedhi_project.footer;
	});

	$rootScope.$watch("templateManager.container", function() {
		footer = templateManager.container ? templateManager.container.footer : footer;
	});

	/**
	* Changes the window to the edit mode
	* @function editMode
	* @memberof ScreenService
	* @instance
	*/
	this.editMode = function() {
		this.defaultFooter();
		$rootScope.$broadcast('ui.editMode');
	};

	/**
	* Changes the window to the view mode
	* @function viewMode
	* @memberof ScreenService
	* @instance
	*/
	this.viewMode = function() {
		$rootScope.$broadcast('ui.viewMode');
	};

	/**
	* Changes the widget to allow multi selection
	* @function allowMultiSelect
	* @memberof ScreenService
	* @instance
	* @param {Widget} widget - widget the should alllow multi selection
	*/
	this.allowMultiSelect = function(widget) {
		$rootScope.$broadcast('ui.allowMultiSelect', widget);
	};

	/**
	* Changes the widget to prevent multi selection
	* @function disableMultiSelect
	* @memberof ScreenService
	* @instance
	* @param {Widget} widget - widget the should alllow multi selection
	*/
	this.disableMultiSelect = function(widget) {
		$rootScope.$broadcast('ui.disableMultiSelect', widget);
	};

	/**
	* Unselects all the widget rows
	* @function deselectAll
	* @memberof ScreenService
	* @instance
	* @param {Widget} widget - Widget to have it's rows unselected
	*/
	this.deselectAll = function(widget) {
		$rootScope.$broadcast('ui.deselectAll', widget);
	};

	/**
	* Closes the field view
	* @function closeFieldView
	* @memberof ScreenService
	* @instance
	*/
	this.closeFieldView = function() {
		delete templateManager.fieldList;
		delete templateManager.container.innerFooter;
		this.editMode();
	};

	/**
	* Closes the footer
	* @function closeFooter
	* @memberof ScreenService
	* @instance
	*/
	this.closeFooter = function() {
		$rootScope.footerOpen = false;
		templateManager.updateTemplate();
	};

	/**
	* Updates the templates on the screen
	* @function updateTemplate
	* @memberof ScreenService
	* @instance
	*/
	this.updateTemplate = function() {
		return templateManager.updateTemplate();
	};
	/**
	* Sets a new footer on the screen
	* @function changeFooterSelect
	* @memberof ScreenService
	* @instance
	* @param {string} footerUrl - URL of the new footer
	*/
	this.changeFooterSelect = function(footerUrl) {
		templateManager.container.innerFooter = footerUrl;
	};

	/**
	* Sets a new footer on the screen
	* @function changeFooter
	* @memberof ScreenService
	* @instance
	* @param {string} footerUrl - URL of the new footer
	*/
	this.changeFooter = function(footerUrl) {
		templateManager.container.footer = footerUrl;
	};

	/**
	* Resets the footer on the screen to the default settings
	* @function defaultFooter
	* @memberof ScreenService
	* @instance
	*/
	this.defaultFooter = function() {
		templateManager.container.footer = footer;
	};

	/**
	* Saves the changes on the current row of the widget and returns to view mode
	* @function saveChange
	* @memberof ScreenService
	* @instance
	* @param {Widget} widget - Widget to be saved
	*/
	this.saveChange = function(widget) {
		var index = widget.currentRow.__index;

		if (index === -1) {
			widget.dataSource.data.push(widget.currentRow);
		} else {
			widget.dataSource.data[widget.currentRow.__index] = widget.currentRow;
		}

		this.viewMode();
	};

	/**
	* Changes widget to view mode and discard changes
	* @function cancelChange
	* @memberof ScreenService
	* @instance
	* @param {Widget} widget - Widget to be saved
	* @returns {Boolean} True if current row has changes
	*/
	this.cancelChange = function(widget) {
		var index = widget.currentRow.__index;

		this.viewMode();

		if (index === -1) {
			widget.currentRow = {};
			return false;
		} else {
			var _row = widget.dataSource.data[widget.currentRow.__index];
			widget.currentRow = angular.copy(_row);
			widget.currentRow.__index = widget.dataSource.data.indexOf(_row);
			return true;
		}
	};

	/**
	* Closes current widget and opens the previous one
	* @function goBack
	* @memberof ScreenService
	* @instance
	* @param {Boolean} removeFromHistory - True if the window being close is not suppose to be registered in the history
	* @returns {Promise} Returns a promise that will succeed when the widget is closed.
	*/
	this.goBack = function(removeFromHistory) {
		/*
		 * ScreenService.goBack() can be called when the user has a select field open.
		 * If that's the case, instead of opening the last window, it should run
		 * ScreenService.closeFieldView(). We detect whether it's open by checking if
		 * templateManager.fieldList is defined. If the user presses cancel, this is never
		 * called, but on the hardware back button, only __back__() is called, which then
		 * comes here.
		 */
		var promise;
		if ($rootScope.swipes.length > 0) {
			// closes the swipe: no args = topmost
			promise = this.closeSwipe();
		} else {
			if (templateManager.historyItems.length > 1) {
				if (!templateManager.project.maintainHistoryItems && !Util.isFalse(removeFromHistory)) {
					templateManager.historyItems.pop();
				}
				var lastHistoryItem = getLastHistoryItem(templateManager);
				$rootScope.openWindow(lastHistoryItem);
				if (templateManager.wizard && templateManager.wizard.stages[lastHistoryItem]) {
					templateManager.wizard.currentStage = templateManager.wizard.stages[lastHistoryItem];
				}
				promise = templateManager.updateTemplate();
			} else {
				promise = ZHPromise.reject();
			}
		}
		return promise;
	};

	function getLastHistoryItem(templateManager) {
		var lastHistoryItem;
		if (templateManager.project.maintainHistoryItems) {
			lastHistoryItem = self.getLastHistoryItemFromContainer(templateManager.container);
		} else {
			lastHistoryItem = templateManager.historyItems.pop();
		}

		return lastHistoryItem;
	}

	/**
	* Retrieves last item from the containers history
	* @function getLastHistoryItemFromContainer
	* @memberof ScreenService
	* @instance
	* @param {Container} container - Container to have history searched
	* @returns {Number} Index of the last history item
	*/
	this.getLastHistoryItemFromContainer = function getLastHistoryItemFromContainer(container) {
		var containerJsonName = Util.getContainerJsonName(container);
		var lastHistoryItemIndex = templateManager.historyItems.indexOf(containerJsonName);
		return templateManager.historyItems[lastHistoryItemIndex - 1];
	};

	/**
	* Retrieves the history items from the window
	* @function getHistory
	* @memberof ScreenService
	* @instance
	* @returns {Array} Array of items in history
	*/
	this.getHistory = function() {
		return templateManager.historyItems;
	};

	/**
	* Clears the history
	* @function clearHistoryItems
	* @memberof ScreenService
	* @instance
	*/
	this.clearHistoryItems = function(){
		templateManager.historyItems = [];
	};

	/**
	* Filter the ajacent widget dataSource using the current value from current widget
	* @function filterWidget
	* @memberof ScreenService
	* @instance
	* @param {Widget} filterWidget - Current widget
	* @param {Array} widgets - The widgets where both the current widget and the one to be filtered are into.
	* @deprecated since version 1.46
	*/
	this.filterWidget = function(filterWidget, widgets) {

		var row = filterWidget.currentRow;
		/*  jszeedhi: poo-ta-ree-a
			Taking the next widget as the one that's supposed to be filtered
			means you can only filter a widget that's adjacent to another.
		*/
		var toBeFiltered = widgets[widgets.indexOf(filterWidget) + 1];
		var filter;

		if (row) {
			filter = [{
				name: filterWidget.valueField,
				operator: '=',
				value: row[filterWidget.valueField]
			}];
		}

		toBeFiltered.dataSource.filter(filter).then(function(filtered) {
			toBeFiltered.dataSource.data = filtered;
		});
	};

	function getIconFromType(type, defaultIcon) {
		var icon = defaultIcon;
		if (type) {
			switch (type.toUpperCase()) {
				case 'ERROR':
				case 'E':
					icon = 'error';
					break;
				case 'ALERT':
				case 'W':
					icon = 'exclamation';
					break;
				case 'INFORMATION':
					icon = 'information';
					break;
				case 'CONFIRMATION':
					icon = 'question';
					break;
				case 'SUCCESS':
				case 'M':
					icon = 'checked';
					break;
			}
		}
		return icon;
	}

	/**
	* Verifies if the alert is shown
	* @function isAlertShown
	* @memberof ScreenService
	* @instance
	*/
	this.isAlertShown = function() {
		return !!$rootScope.alertShown;
	};

	/**
	* Shows a customized dialog
	* @function showCustomDialog
	* @memberof ScreenService
	* @instance
	* @param {String} message - Message to be shown
	* @param {String} type - Type of the dialog to be shown
	* @param {String} icon - Icon to be show on the dialog
	* @param {Array} actions - Actions to be shown in the dialog
	* @param {Object} variables - variables of the message to be translated
	* @returns {Promise} Promise to show the dialog and sends the event to the resolve function
	*/
	this.showCustomDialog = function(message, type, icon, actions, variables){
		var deferred = ZHPromise.defer();
		icon = icon ? icon : getIconFromType(type, 'exclamation');
		$rootScope.$broadcast("showCustomDialog", {
			message: this.i18n(message, variables),
			icon: icon,
			actions: actions,
			deferred: deferred
		});
		return deferred;
	};

	/**
	* Shows a dialog window with a message
	* @function showMessage
	* @memberof ScreenService
	* @instance
	* @param {String} message - Message to be shown
	* @param {String} type - Type of the dialog to be shown
	* @param {String} icon - Icon to be show on the dialog
	* @param {Object} variables - variables of the message to be translated
	* @returns {Promise} Promise to show the dialog and sends the event to the resolve function
	*/
	this.showMessage = function(message, type, icon, variables) {
		return this.showCustomDialog(message, type, icon, [{
			label: this.i18n("OK"),
			"default": true,
			code: function(e, deferred) {
				deferred.resolve(e);
			}
		}], variables);
	};

	/**
	* Shows a confirmation dialog window
	* @function confirmMessage
	* @memberof ScreenService
	* @instance
	* @param {String} message - Message to be shown
	* @param {String} type - Type of the dialog to be shown
	* @param {Function} success - Function to be executed on confirmation
	* @param {Function} fail - Function to be executed on canceling
	* @param {String} icon - Icon to be show on the dialog
	* @param {Object} variables - variables of the message to be translated
	* @returns {Promise} Promise to show the dialog and sends the event to the resolve or to the reject
	*/
	this.confirmMessage = function(message, type, success, fail, icon, variables) {
		icon = icon ? icon : getIconFromType(type, 'question');
		return this.showCustomDialog(message, type, icon, [{
			label: this.i18n("Yes"),
			"default": true,
			code: function(e, deferred) {
				if (Util.isFunction(success)) {
					success(e);
				}
				deferred.resolve(e);
			}
		}, {
			label: this.i18n("No"),
			code: function(e, deferred) {
				if (Util.isFunction(fail)) {
					fail(e);
				}
				deferred.reject(e);
			}
		}], variables);
	};

	/**
	* Shows an alert notification
	* @function alertNotification
	* @memberof ScreenService
	* @instance
	* @param {String} message - Message to be shown
	* @param {Number} time - Time to be kept on screen
	* @param {Object} variables - variables of the message to be translated
	*/
	this.alertNotification = function(message, time, variables) {
		this.notificationMessage(message, 'alert', time, variables);
	};

	/**
	* Shows a success notification
	* @function successNotification
	* @memberof ScreenService
	* @instance
	* @param {String} message - Message to be shown
	* @param {Number} time - Time to be kept on screen
	* @param {Object} variables - variables of the message to be translated
	*/
	this.successNotification = function(message, time, variables) {
		this.notificationMessage(message, 'success', time, variables);
	};

	/**
	* Shows a error notification
	* @function errorNotification
	* @memberof ScreenService
	* @instance
	* @param {String} message - Message to be shown
	* @param {Number} time - Time to be kept on screen
	* @param {Object} variables - variables of the message to be translated
	*/
	this.errorNotification = function(message, variables) {
		this.notificationMessage(message, 'error', undefined, variables);
	};

	/**
	* Shows a notification on the screen
	* @function notificationMessage
	* @memberof ScreenService
	* @instance
	* @param {String} message - Message to be shown
	* @param {String} type - Type of the notification
	* @param {Number} time - Time to be kept on screen
	* @param {Object} variables - variables of the message to be translated
	*/
	this.notificationMessage = function(message, type, time, variables) {
		$rootScope.$broadcast("addNotificationMessage", {
			text: this.i18n(message, variables),
			type: type,
			time: time
		});
	};

	/**
	* Cancels the current filter
	* @function cancelFilter
	* @memberof ScreenService
	* @instance
	*/
	this.cancelFilter = function() {
		this.closeFooter();
		delete $rootScope._currentFilter;
	};

	/**
	* Opens the widget as a gadget
	* @function cancelFilter
	* @memberof ScreenService
	* @instance
	* @param {Widget} widget - Widget to be opened
	* @param {String} type - Type of gadget to open the window
	*/
	this.openWidgetAsGadget = function(widget, type) {
		switch (type) {
			case this.FILTER:
				this.changeFilter(widget);
				break;
			case this.SWIPE:
				this.openSwipe(widget);
				break;
			case this.POPUP:
				this.openPopup(widget);
				break;
			default:
		}
	};

	/**
	* Opens the filter
	* @function changeFilter
	* @memberof ScreenService
	* @instance
	* @param {Widget} widget - Widget to be filtered
	*/
	this.changeFilter = function(widget) {
		$rootScope.$broadcast("openFilter", widget);
		$rootScope._currentFilter = widget;
		$rootScope.footerOpen = true;
	};

	/**
	* Toogle the filter popup
	* @function toogleFilter
	* @memberof ScreenService
	* @instance
	* @param {Widget} widget - Widget to be opened or closed
	* @param {String} type - Type of gadget to open the window
	*/
	this.toggleFilter = function(widget) {
		if ($rootScope.footerOpen) {
			this.cancelFilter();
		} else {
			this.changeFilter(widget);
		}
	};

	/**
	* Opens a widget in a popup
	* @function openPopup
	* @memberof ScreenService
	* @instance
	* @param {Widget} widget - Widget to be opened
	*/
	this.openPopup = function(widget) {
		var deferred = ZHPromise.defer();
		//widget.lastActiveWidget = templateManager.container.getActiveWidget();
		widget.activate();
		$rootScope.$broadcast("openPopup", widget, deferred);

		return deferred.promise.then(function() {
			if (!$rootScope._popupStack) {
				$rootScope._popupStack = [];
			}
			$rootScope._popupStack.push(widget);
			$rootScope._currentPopup = widget;
		});

	};

	/**
	* Retrieves opened popup
	* @function popupOpen
	* @memberof ScreenService
	* @instance
	* @returns {Widget} Widget opened in the popup
	*/
	this.popupOpen = function() {
		return $rootScope.popupOpen;
	};

	/* param closeAll - closes all opened popups */
	/**
	* Closes opened popups
	* @function closePopup
	* @memberof ScreenService
	* @instance
	* @param {Boolean} closeAll - If true close all the popups
	*/
	this.closePopup = function(closeAll) {
		if ($rootScope._currentPopup) {
			$rootScope._currentPopup.restoreLastActiveWidget();
		}
		if ($rootScope.popupOpen && $rootScope._popupStack && $rootScope._popupStack.length > 0) {
			if (closeAll === true) {
				for (var i = ($rootScope._popupStack.length - 1); i >= 0; i--) {
					if ($rootScope._popupStack[i] && Util.isFunction($rootScope._popupStack[i].popupOnCancel)) {
						$rootScope._popupStack[i].popupOnCancel();
					}
				}
				$rootScope._popupStack = [];
			} else {
				if (Util.isFunction($rootScope._currentPopup.popupOnCancel)) {
					$rootScope._currentPopup.popupOnCancel();
				}
				$rootScope._popupStack.pop();
			}

			if ($rootScope._popupStack.length === 0) {
				delete $rootScope._currentPopup;
				$rootScope.popupOpen = false;
				$rootScope.$broadcast("closePopup", $rootScope._currentPopup);
			} else {
				$rootScope._currentPopup = $rootScope._popupStack[$rootScope._popupStack.length - 1];
				$rootScope.$broadcast("openPopup", $rootScope._currentPopup);
			}

			templateManager.updateTemplate();
		}
	};

	/**
	* Verifies if there is any opened popup
	* @function isPopupOpen
	* @memberof ScreenService
	* @instance
	*/
	this.isPopupOpen = function() {
		return !!$rootScope.popupOpen;
	};

	$rootScope.isPopupOpen = this.isPopupOpen;

	/**
	* Retrieves opened popup
	* @function getPopupOpen
	* @memberof ScreenService
	* @instance
	* @returns {Widget} Widget opened in the popup
	*/
	this.getPopupOpen = function() {
		return this.isPopupOpen() ? $rootScope._currentPopup : undefined;
	};

	/**
	* Sets the user data on the project
	* @function buildUserData
	* @memberof ScreenService
	* @instance
	* @param {String} userName - Name of the user
	* @param {Array} descriptions - (Opcional) Array of user descriptions
	* @param {Function} clickFunc - Function to be executed on clicking the profile
	* @param {url} userImage - User image
	*/
	this.buildUserData = function(userName, descriptions, clickFunc, userImage) {
		templateManager.userData = {};
		templateManager.showUserData = true;
		templateManager.userData.descriptions = [];
		templateManager.userData.name = userName;
		templateManager.userData.image = userImage || templateManager.defaultUserImageUrl || "./images/user.jpg";
		descriptions.forEach(function(text) {
			templateManager.userData.descriptions.push(text);
		});

		templateManager.clickUserData = function() {
			self.toggleSideMenu();
			if (clickFunc) {
				Util.doAsync(clickFunc);
			}
		};
	};

	/**
	* Sets information on the header
	* @function setHeaderInfo
	* @memberof ScreenService
	* @instance
	* @param {String} info - Information to be set on header
	*/
	this.setHeaderInfo = function(info) {
		templateManager.headerInfo = info;
		templateManager.updateTemplate();
	};

	/**
	* Opens a new container
	* @function openWindow
	* @memberof ScreenService
	* @instance
	* @param {String} containerName - Name of the container to be opened
	* @param {Boolean} maintainWindowFilter - True if filters should be maintained
	* @returns {Promise} Returns a promise to open a new window
	*/
	this.openWindow = function(containerName, maintainWindowFilter) {
		return $rootScope.openWindow(containerName, maintainWindowFilter);
	};

	/**
	* Opens a new link
	* @function openLink
	* @memberof ScreenService
	* @instance
	* @param {String} url - Url to be opened
	* @param {Boolean} newWindow - True if the url should be opened on a new window (tab)
	*/
	this.openLink = function(url, newWindow) {
		$rootScope.openLink(url, newWindow);
	};

	/**
	* Opens the widget in a swipe
	* @function openSwipe
	* @memberof ScreenService
	* @instance
	* @param {Widget} widget - Widget to be opened
	* @param {Boolean} closeAll - True if all other swipes should be closed
	* @param {Class} customClass - Class with custom methods to be inserted in the method
	* @returns {Promise} Promise to open the swipe and provides the opened widget to success or a error message to reject
	*/
	this.openSwipe = function(widget, closeAll, customClass) {
		var deferred = ZHPromise.defer();
		$rootScope.$broadcast("openSwipe", widget, closeAll, deferred, customClass);
		return deferred.promise.then(function() {
			widget.activate();
			var previousActiveWidget = widget.lastActiveWidget;
			if (previousActiveWidget.updateSwipeTip) {
				previousActiveWidget.updateSwipeTip(previousActiveWidget.currentRow);
			}
		});
	};

	/**
	* closes the swipe
	* @function closeSwipe
	* @memberof ScreenService
	* @instance
	* @param {Number} level - 0 closes all swipes, undefined closes the topmost swipe, more than 0 closes all swipes bigger than level
	* @returns {Promise} Promise to closing the swipe and provides the closed widget to success or a error message to reject
	*/
	this.closeSwipe = function(level) {
		var deferred = ZHPromise.defer();

		$rootScope.$broadcast("closeSwipe", level, deferred);

		return deferred.promise;
	};

	/**
	* Verifies if any swipe is opened above the level
	* @function isSwipeOpen
	* @memberof ScreenService
	* @instance
	* @param {Number} swipeLevel - (Optional) Base level of the swipes. If undefined level will be 0
	* @returns {Boolean} True if any swipe opened above level specified
	*/
	this.isSwipeOpen = function(swipeLevel) {
		if (swipeLevel === undefined) {
			swipeLevel = 0;
		}
		return $rootScope.swipes.length > swipeLevel;
	};

	/**
	* Verifies if any editable swipe is opened
	* @function isEditableSwipeOpened
	* @memberof ScreenService
	* @instance
	* @returns {Boolean} True if any editable swipe opened
	*/
	this.isEditableSwipeOpened = function() {
		var result = false;
		if ($rootScope.swipes[0]) {
			result = $rootScope.swipes[0].container.mode === "edit";
		}
		return result;
	};

	/**
	* Verifies if widget is opened as a swipe
	* @function isWidgetAsSwipe
	* @memberof ScreenService
	* @instance
	* @param {Widget} widget - Widget to be verified
	* @returns {Boolean} True if widget is opened as swipe
	*/
	this.isWidgetAsSwipe = function(widget) {
		return $rootScope.swipes.indexOf(widget) !== -1;
	};

	/**
	* Retrieves the level where we can find the swipe
	* @function getSwipeLevel
	* @memberof ScreenService
	* @instance
	* @param {Widget} widget - Swipe widget
	* @returns {Number} Level of the swipe
	*/
	this.getSwipeLevel = function(widget) {
		var widgetLevel = $rootScope.swipes.indexOf(widget);
		if (widgetLevel === -1 && widget.parent) {
			return this.getSwipeLevel(widget.parent);
		}
		return widgetLevel;
	};

	$rootScope.isSwipeOpen = this.isSwipeOpen;
	$rootScope.getSwipeLevel = this.getSwipeLevel;
	$rootScope.isEditableSwipeOpened = this.isEditableSwipeOpened;

	/**
	* @typedef {Object} Notification
	* @property {String} text The text of the notification
	* @property {String} type The type of the notification
	* @property {Number} time The time that the notification should be shown on the screen
	*/

	/**
	* Retrieve a notification specified
	* @function getNotification
	* @memberof ScreenService
	* @instance
	* @param {String} notification - name of the notification to be retrieved
	* @returns {Notification} The specified notification
	*/
	this.getNotification = function(notification) {
		var notifications = templateManager.project.notifications;
		var notificationObj = $.grep(notifications, function(item) {
			return item.name == notification || item.id == notification;
		});
		return notificationObj[0];
	};

	/*
	* Set up a hint for a  notification
	* @function setNotificationHint
	* @memberof ScreenService
	* @instance
	* @param {String} notification - Name of the notification
	* @param {String} hint - Hint to be setted in the notification
	*/
	this.setNotificationHint = function(notification, hint) {
		var notificationObj = this.getNotification(notification);
		if (notificationObj) {
			notificationObj.hint = hint;
		}
	};

	/*
	* Changes the loading message of the project
	* @function changeLoadingMessage
	* @memberof ScreenService
	* @instance
	* @param {String} message - New loading message
	*/
	this.changeLoadingMessage = function(message) {
		templateManager.loadingMessage = message;
		templateManager.onUpdate();
	};

	/*
	* Restores the default loading message of the project
	* @function restoreDefaultLoadingMessage
	* @memberof ScreenService
	* @instance
	*/
	this.restoreDefaultLoadingMessage = function(){
		templateManager.loadingMessage = undefined;
		templateManager.onUpdate();
	};

	/*
	* Hides the loader
	* @function hideLoader
	* @memberof ScreenService
	* @instance
	*/

	this.hideLoader = function() {
		$rootScope.$broadcast("request.stop");
	};

	/*
	* Shows the loader
	* @function showLoader
	* @memberof ScreenService
	* @instance
	*/
	this.showLoader = function() {
		$rootScope.$broadcast("request.start");
	};

	/*
	* Changes the menu between opened and closed states
	* @function toogleSideMenu
	* @memberof ScreenService
	* @instance
	*/
	this.toggleSideMenu = function() {
		if (templateManager.container.showMenu && !$rootScope.isEditableSwipeOpened()) {
			$rootScope.menuOpen = !$rootScope.menuOpen;
			templateManager.updateTemplate();
		}
	};

	/*
	* Verifies is side menu is open
	* @function isSideMenuOpen
	* @memberof ScreenService
	* @instance
	* @returns {Boolean} True if menu is open
	*/
	this.isSideMenuOpen = function() {
		return $rootScope.menuOpen;
	};

	/*
	* Verifies if keyboard is open
	* @function isKeyboardOpen
	* @memberof ScreenService
	* @instance
	* @returns {Boolean} True if keyboard is open
	*/
	this.isKeyboardOpen = function() {
		return window.isKeyboardOpened;
	};

	/*
	* Allows widget to be edited
	* @function startEditWidget
	* @memberof ScreenService
	* @instance
	* @param {Widget} widget - Widget to be edited
	*/
	this.startEditWidget = function(widget) {
		$rootScope.widgetEditing = widget;
		templateManager.updateTemplate();
	};

	/*
	* Prevent widget to be edited
	* @function stopEditWidget
	* @memberof ScreenService
	* @instance
	*/
	this.stopEditWidget = function() {
		$rootScope.widgetEditing = undefined;
		templateManager.updateTemplate();
	};

	/*
	* Verifies is widget editing is currently allowed
	* @function isWidgetEditing
	* @memberof ScreenService
	* @instance
	* @returns {Boolean} True if edit is currently allowed
	*/
	this.isWidgetEditing = function() {
		return !Util.isUndefined($rootScope.widgetEditing);
	};

	/*
	* Shows information about the project
	* @function showAbout
	* @memberof ScreenService
	* @instance
	*/
	this.showAbout = function(){
		if (this.isSideMenuOpen()) this.toggleSideMenu();
		$rootScope.$broadcast("showAbout");
	};

	/*
	* Hides information about the project
	* @function hideAbout
	* @memberof ScreenService
	* @instance
	*/
	this.hideAbout = function(){
		$rootScope.$broadcast("hideAbout");
	};

	/*
	* Verifies if information about the project is being shown
	* @function isAboutShown
	* @memberof ScreenService
	* @instance
	*/
	this.isAboutShown = function() {
		return !!$rootScope.showAbout;
	};

	/*
	* Downloads a file from the url
	* @function downloadFileFromURL
	* @memberof ScreenService
	* @instance
	* @param {String} url - Addres of the file to be downloaded
	* @param {String} filename - (Optional) Name to be given to the downloaded file. If undefined, will use the same name as the source file
	*/
	this.downloadFileFromURL = function(url, filename) {
		filename = filename || url.substring(url.lastIndexOf("/") + 1).split("?")[0];
		var a = document.createElement('a');

		a.href = url;
		a.download = filename;
		a.style.display = 'none';
		document.body.appendChild(a);
		a.click();
		document.body.removeChild(a);
	};
}